# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .organizations_preview_subscription_change_response_validation_errors_item_resource import (
    OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItemResource,
)


class OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItem(
    UniversalBaseModel
):
    resource: OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItemResource
    current_usage: typing_extensions.Annotated[
        float, FieldMetadata(alias="currentUsage")
    ]
    limit: float
    message: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
