# This file was auto-generated by Fern from our API Definition.

import typing

OrganizationInvitationsGetResponseStatus = typing.Union[
    typing.Literal["pending", "accepted", "revoked"], typing.Any
]
