# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.date import Date
from .organizations_subscription_response_scheduled_change_target_plan import (
    OrganizationsSubscriptionResponseScheduledChangeTargetPlan,
)


class OrganizationsSubscriptionResponseScheduledChange(UniversalBaseModel):
    target_plan: typing_extensions.Annotated[
        OrganizationsSubscriptionResponseScheduledChangeTargetPlan,
        FieldMetadata(alias="targetPlan"),
    ]
    effective_date: typing_extensions.Annotated[
        Date, FieldMetadata(alias="effectiveDate")
    ]
    schedule_id: typing_extensions.Annotated[str, FieldMetadata(alias="scheduleId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
