# This file was auto-generated by Fern from our API Definition.

import typing

ProjectMembershipsListResponseItemRole = typing.Union[
    typing.Literal["ADMIN", "DEVELOPER", "VIEWER", "ANNOTATOR"], typing.Any
]
