# This file was auto-generated by Fern from our API Definition.

import typing

TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue = typing.Union[
    str, float
]
