# This file was auto-generated by Fern from our API Definition.

import typing

OrganizationsUpdateSubscriptionRequestTargetPlan = typing.Union[
    typing.Literal["free", "pro", "team"], typing.Any
]
