# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .plan_limit_exceeded_error_tag import PlanLimitExceededErrorTag


class PlanLimitExceededError(UniversalBaseModel):
    message: str
    resource: str
    limit_type: typing_extensions.Annotated[str, FieldMetadata(alias="limitType")]
    current_usage: typing_extensions.Annotated[
        float, FieldMetadata(alias="currentUsage")
    ]
    limit: float
    plan_tier: typing_extensions.Annotated[str, FieldMetadata(alias="planTier")]
    tag: PlanLimitExceededErrorTag

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
