# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .traces_create_otel_request_resource_spans_item_resource import (
    TracesCreateOtelRequestResourceSpansItemResource,
)
from .traces_create_otel_request_resource_spans_item_scope_spans_item import (
    TracesCreateOtelRequestResourceSpansItemScopeSpansItem,
)


class TracesCreateOtelRequestResourceSpansItem(UniversalBaseModel):
    resource: typing.Optional[TracesCreateOtelRequestResourceSpansItemResource] = None
    scope_spans: typing_extensions.Annotated[
        typing.List[TracesCreateOtelRequestResourceSpansItemScopeSpansItem],
        FieldMetadata(alias="scopeSpans"),
    ]
    schema_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="schemaUrl")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
