# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.date import Date


class OrganizationsUpdateSubscriptionResponse(UniversalBaseModel):
    requires_payment: typing_extensions.Annotated[
        bool, FieldMetadata(alias="requiresPayment")
    ]
    client_secret: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="clientSecret")
    ] = None
    scheduled_for: typing_extensions.Annotated[
        typing.Optional[Date], FieldMetadata(alias="scheduledFor")
    ] = None
    schedule_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="scheduleId")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
