# This file was auto-generated by Fern from our API Definition.

import typing

TracesSearchByEnvRequestSortBy = typing.Union[
    typing.Literal["start_time", "duration_ms", "total_tokens"], typing.Any
]
