# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.date import Date
from .organizations_preview_subscription_change_response_validation_errors_item import (
    OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItem,
)


class OrganizationsPreviewSubscriptionChangeResponse(UniversalBaseModel):
    is_upgrade: typing_extensions.Annotated[bool, FieldMetadata(alias="isUpgrade")]
    prorated_amount_in_dollars: typing_extensions.Annotated[
        float, FieldMetadata(alias="proratedAmountInDollars")
    ]
    next_billing_date: typing_extensions.Annotated[
        Date, FieldMetadata(alias="nextBillingDate")
    ]
    recurring_amount_in_dollars: typing_extensions.Annotated[
        float, FieldMetadata(alias="recurringAmountInDollars")
    ]
    can_downgrade: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="canDowngrade")
    ] = None
    validation_errors: typing_extensions.Annotated[
        typing.Optional[
            typing.List[
                OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItem
            ]
        ],
        FieldMetadata(alias="validationErrors"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
