# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rate_limit_error_tag import RateLimitErrorTag


class RateLimitError(UniversalBaseModel):
    message: str
    organization_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="organizationId")
    ]
    limit: float
    retry_after: typing_extensions.Annotated[float, FieldMetadata(alias="retryAfter")]
    plan_tier: typing_extensions.Annotated[str, FieldMetadata(alias="planTier")]
    tag: RateLimitErrorTag

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
