# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .traces_create_request_resource_spans_item_scope_spans_item_spans_item_attributes_item_value_array_value import (
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue,
)
from .traces_create_request_resource_spans_item_scope_spans_item_spans_item_attributes_item_value_int_value import (
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue,
)
from .traces_create_request_resource_spans_item_scope_spans_item_spans_item_attributes_item_value_kvlist_value import (
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue,
)


class TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue(
    UniversalBaseModel
):
    string_value: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="stringValue")
    ] = None
    int_value: typing_extensions.Annotated[
        typing.Optional[
            TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue
        ],
        FieldMetadata(alias="intValue"),
    ] = None
    double_value: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="doubleValue")
    ] = None
    bool_value: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="boolValue")
    ] = None
    array_value: typing_extensions.Annotated[
        typing.Optional[
            TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue
        ],
        FieldMetadata(alias="arrayValue"),
    ] = None
    kvlist_value: typing_extensions.Annotated[
        typing.Optional[
            TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue
        ],
        FieldMetadata(alias="kvlistValue"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
