# This file was auto-generated by Fern from our API Definition.

import typing

OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItemResource = (
    typing.Union[typing.Literal["seats", "projects"], typing.Any]
)
