# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .traces_create_otel_request_resource_spans_item_scope_spans_item_spans_item_attributes_item_value import (
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue,
)


class TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItem(
    UniversalBaseModel
):
    key: str
    value: TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
